/***************************************************************************/
/*                        TQuickPrint version 1.01                         */
/* ----------------------------------------------------------------------- */
/* What is it: Ever needed to print some text quickly but was tired of     */
/* ----------- the lot of work it requires to use TPrinter?                */
/*             On top of that it takes a lot of code to make your prints   */
/*             look good and look good with different printer resolutions. */
/*                                                                         */
/*             TQuickPrint is the answer for those "problems".             */
/*                                                                         */
/*                                                                         */
/* How to install it: Go to the "Component" Menuitem and select "Install   */
/* ------------------ Component...". Click the "Browse" button next to the */
/*                    "Unit file name:". Choose "QuickPrint.cpp".          */
/*                    Choose the apporpriate package to install TQuickPrint*/
/*                    in to, from the "Package file name" combobox         */
/*                    (normally: dclusr35.bpk or dclusr40.bpk)             */
/*                    TQuickPrint will register it self in the "System"    */
/*                    palette.                                             */
/*                                                                         */
/* How to use it: Just drop it on to the Form you're working with and work */
/* -------------- on.                                                      */
/*                                                                         */
/* What does it cost: Nothing. It's free of charge for any usage! :-)      */
/* ------------------                                                      */
/*                                                                         */
/* Which version of C++ Builder is it tested with: 3 and 4. Should work    */
/* ----------------------------------------------- with version 1 too.     */
/*                                                                         */
/* Disclaimer:                                                             */
/* -----------                                                             */
/* TQuickPrint is provided "AS IS" without warranty of any                 */
/* kind, either expressed or implied, including but not limited            */
/* to the implied warranties of merchantability and fitness                */
/* for a particular purpose. In no event shall Cramon Utilites             */
/* be liable for any damages whatsoever including direct, indirect,        */
/* incidental, consequential, loss of business profits or                  */
/* special damages, even if Cramon Utilities has been advised              */
/* of the possibility of such damages.                                     */
/* You may use TQuickPrint free of charge in any application you make      */
/* be it freeware, shareware or commercial. If you change the sourcecode   */
/* of TQuickPrint you must include this text and clearly state which parts */
/* are changed by you and that this is not the original version.           */
/* If you would like your changes in the "official" version please send    */
/* the updated source file(s) to me and I'll update my sourcecode (you'll  */
/* ofcourse get credit for all changes you've made to TQuickPrint).        */
/*                                                                         */
/* Author..: Jeppe Cramon                                                  */
/* Company.: Cramon Utilities                                              */
/* WWW.....: http://www.cramon.dk                                          */
/* E-mail..: jeppe@cramon.dk                                               */
/* (C) Copyright 1999 Cramon Utilities                                     */
/*                                                                         */
/* ----------------------------------------------------------------------- */
/* Date         Comment                                 Programmer         */
/* 05/30-1999   Created the first version (1.00)        Jeppe Cramon       */
/* 06/05-1999   Version (1.01)                          Jeppe Cramon       */
/*              Fixed the Convertion functions so they                     */
/*              work properly (caused by casting problems).                */
/*              Fixed all Printing routines so the                         */
/*              Margins now are correct (I hope) ;)                        */
/*              Changed the consts in the header, to                       */
/*              avoid problems when compiling together                     */
/*              with THintBox.                                             */
/***************************************************************************/

//---------------------------------------------------------------------------
#ifndef quickprintH
#define quickprintH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include <printers.hpp>
//---------------------------------------------------------------------------
// Enums
enum TMeasureUnit { Inches, Millimeters };
enum TLineSpacing { HalfSpace, SingleSpace, SingleAndAHalf, DoubleSpace };

//---------------------------------------------------------------------------
class PACKAGE TQuickPrint : public TComponent
{
    private :
        // Private vars that are used as properties
        bool                bWordWrap;              // Print using WordWrap
        TMeasureUnit        fMeasureUnit;           // Measure Unit (Inches, Millimeters)
        TLineSpacing        fLineSpacing;           // Line Spacing
        Classes::TStrings*  fLines;                 // Text lines
        float               fMarginTop;             // Top margin
        float               fMarginBottom;          // Bottom margin
        float               fMarginLeft;            // Left margin
        float               fMarginRight;           // Right margin
        AnsiString          asPrintTitle;           // Title of printjob
        bool                bPrintPageNo;           // Print the page number
        unsigned int        uiLines;                // Line counter
        unsigned int        uiPageNumber;           // Page counter

        // Events
    	Classes::TNotifyEvent fOnBeginDoc;
    	Classes::TNotifyEvent fOnEndDoc;
    	Classes::TNotifyEvent fOnAbortDoc;
    	Classes::TNotifyEvent fOnNewLine;
	    Classes::TNotifyEvent fOnNewPage;

        // Private vars in general
        int                 iLineHeight;            // Height of a line
        unsigned int        uiX, uiY;               // Pen position

        // Vars used for Parsing HTML text
        TStringList*        TextList;               // Contains HTML text without Align tags
        TList*              AlignList;              // Contains ALIGN information (see const section for info on aling values)

        // Private functions
        int __fastcall FindNext(AnsiString asText, AnsiString asFind, int iOldIndex);

    protected :
        // Property related functions
        void __fastcall SetWordWrap(bool Value);
        void __fastcall SetMeasureUnit(TMeasureUnit Value);
        void __fastcall SetLineSpacing(TLineSpacing Value);
        void __fastcall SetLines(Classes::TStrings* Value);
        void __fastcall SetMarginTop(float Value);
    	void __fastcall SetMarginBottom(float Value);
	    void __fastcall SetMarginLeft(float Value);
    	void __fastcall SetMarginRight(float Value);
        void __fastcall SetPrintTitle(AnsiString Value);
        void __fastcall SetPrintPageNo(bool Value);
        Printers::TPrinterOrientation __fastcall GetOrientation(void);
        void __fastcall SetOrientation(Printers::TPrinterOrientation Value);
        void __fastcall SetFont(Graphics::TFont* Value);
        TFont* __fastcall GetFont();
        float __fastcall GetPhysicalOffsetX();
        float __fastcall GetPhysicalOffsetY();

        // Public Property functions
        bool  __fastcall GetPrinting();
        float __fastcall GetPageHeight();
        float __fastcall GetPageWidth();
        float __fastcall GetPhysicalPageHeight();
        float __fastcall GetPhysicalPageWidth();
        Graphics::TCanvas* __fastcall GetCanvas(void);


        // HTML Parsing (made virtual so they can be enhanced and extended by the user)
        void virtual __fastcall ParseLines();
        void virtual __fastcall ParseLine(AnsiString asText);
        bool virtual __fastcall OnlyContainsTags(AnsiString asText);

        // HTML Print (made virtual so they can be enhanced and extended by the user)
        TSize virtual __fastcall TextExtentEx(AnsiString asTestText);
        void virtual __fastcall TextOutEx(int X, int Y, const AnsiString asTextOut);

    public :
        __fastcall virtual TQuickPrint(TComponent* Owner);              // Constructor
        __fastcall virtual ~TQuickPrint();                              // Destructor

        // Printing functions
        void __fastcall BeginDoc();
        void __fastcall EndDoc();
        void __fastcall AbortDoc();
        void __fastcall NewPage();
        void __fastcall PrintLines();                                   // Print the contents of fLines
        void __fastcall PrintLine(AnsiString asText, int iAlignListIndex);// Print one line of text
        void __fastcall Print(float fX, float fY, AnsiString asText); // fX = -1 means that asText contains HTML code that will calculate the XPos automatically (eg. <center>)

        // Helper functions
        float __fastcall PixelsToMeasureUnitHorz(unsigned int Pixels);
        float __fastcall PixelsToMeasureUnitVert(unsigned int Pixels);
        unsigned int   __fastcall MeasureUnitToPixelsHorz(float Measure);
        unsigned int   __fastcall MeasureUnitToPixelsVert(float Measure);

        // Properties
        __property bool Printing             = {read=GetPrinting , nodefault};
        __property Graphics::TCanvas* Canvas = {read=GetCanvas};
    	__property unsigned int LineNumber   = {read=uiLines     , nodefault};
	    __property unsigned int PageNumber   = {read=uiPageNumber, nodefault};
    	__property float PageHeight          = {read=GetPageHeight};
    	__property float PageWidth           = {read=GetPageWidth};
    	__property float PhysicalPageHeight  = {read=GetPhysicalPageHeight};
    	__property float PhysicalPageWidth   = {read=GetPhysicalPageWidth};
    	__property float PhysicalOffsetX     = {read=GetPhysicalOffsetX};
    	__property float PhysicalOffsetY     = {read=GetPhysicalOffsetY};


    __published:

        // --- Properties ---------------------------------------------------
        __property bool WordWrap            = {read=bWordWrap    , write=SetWordWrap   , default = true};
	    __property TMeasureUnit MeasureUnit = {read=fMeasureUnit , write=SetMeasureUnit, default=Millimeters};
    	__property TLineSpacing LineSpacing = {read=fLineSpacing , write=SetLineSpacing, default=SingleSpace};
        __property Classes::TStrings* Lines = {read=fLines       , write=SetLines};
    	__property float MarginTop          = {read=fMarginTop   , write=SetMarginTop};
	    __property float MarginBottom       = {read=fMarginBottom, write=SetMarginBottom};
    	__property float MarginLeft         = {read=fMarginLeft  , write=SetMarginLeft};
	    __property float MarginRight        = {read=fMarginRight , write=SetMarginRight};
	    __property AnsiString PrintTitle    = {read=asPrintTitle , write=SetPrintTitle};
        __property bool PrintPageNo         = {read=bPrintPageNo , write=SetPrintPageNo , default = false};
    	__property Printers::TPrinterOrientation PrintOrientation = {read=GetOrientation, write=SetOrientation, default=0};
        __property Graphics::TFont* Font    = {read=GetFont        , write=SetFont};

        // Event properties
        __property Classes::TNotifyEvent OnBeginDoc = {read=fOnBeginDoc, write=fOnBeginDoc};
        __property Classes::TNotifyEvent OnEndDoc   = {read=fOnEndDoc  , write=fOnEndDoc};
    	__property Classes::TNotifyEvent OnAbortDoc = {read=fOnAbortDoc, write=fOnAbortDoc};
	    __property Classes::TNotifyEvent OnNewLine  = {read=fOnNewLine , write=fOnNewLine};
    	__property Classes::TNotifyEvent OnNewPage  = {read=fOnNewPage , write=fOnNewPage};
};
//---------------------------------------------------------------------------
#endif
